#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <mach/mach.h>
#include <sys/types.h>
#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>

io_iterator_t added_iter;
io_iterator_t removed_iter;

int connected_state = 0;
UInt16 product_id, vendor_id;

IONotificationPortRef remove_notify_port;

void listen_for_device_removal();

void device_added(void *user_info, io_iterator_t iter)
{
    SInt32 score;
    io_iterator_t interface_iter;
    io_service_t interface, usb_device;
    IOCFPlugInInterface **pii;
    IOUSBDeviceInterface **dev;
    IOUSBFindInterfaceRequest req;

    while ((usb_device = IOIteratorNext(iter))) {
        IOCreatePlugInInterfaceForService(usb_device, kIOUSBDeviceUserClientTypeID,
            kIOCFPlugInInterfaceID, &pii, &score);
        IOObjectRelease(usb_device);
        (*pii)->QueryInterface(pii, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
            (LPVOID *)&dev);
        (*pii)->Release(pii);

        req.bInterfaceClass = 0x08; /* mass storage class */
        req.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
        req.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
        req.bAlternateSetting = kIOUSBFindInterfaceDontCare;

        (*dev)->CreateInterfaceIterator(dev, &req, &interface_iter);

        while ((interface = IOIteratorNext(interface_iter))) {
            (*dev)->GetDeviceVendor(dev, &vendor_id);
            (*dev)->GetDeviceProduct(dev, &product_id);

            IOObjectRelease(interface);

            listen_for_device_removal();

            fprintf(stderr, "Device found: vendor $%04x, product $%04x!\n",
                vendor_id, product_id);

            sleep(5);   /* give it a bit of time to mount: ugly hack */

            system("/usr/bin/atvrhdctrl.sh connect");
        }

        (*dev)->Release(dev);
    }
}

void device_removed(void *user_info, io_iterator_t iter)
{
    io_service_t usb_device;

    while ((usb_device = IOIteratorNext(iter))) {
        IONotificationPortDestroy(remove_notify_port);
        IOObjectRelease(usb_device);

        fprintf(stderr, "Device removed!\n");
        system("/usr/bin/atvrhdctrl.sh disconnect");
    }
}

void listen_for_device_removal()
{
    SInt32 s32_vendor_id, s32_product_id;
    mach_port_t master_port;
    CFMutableDictionaryRef matching_dict;
    CFRunLoopSourceRef run_loop_src;

    IOMasterPort(MACH_PORT_NULL, &master_port);

    s32_vendor_id = vendor_id;
    s32_product_id = product_id;

    matching_dict = IOServiceMatching(kIOUSBDeviceClassName);
    CFDictionarySetValue(matching_dict, CFSTR(kUSBVendorName), CFNumberCreate(
        kCFAllocatorDefault, kCFNumberSInt32Type, &s32_vendor_id));
    CFDictionarySetValue(matching_dict, CFSTR(kUSBProductName), CFNumberCreate(
        kCFAllocatorDefault, kCFNumberSInt32Type, &s32_product_id));

    remove_notify_port = IONotificationPortCreate(master_port);
    run_loop_src = IONotificationPortGetRunLoopSource(remove_notify_port);
    CFRunLoopAddSource(CFRunLoopGetCurrent(), run_loop_src, kCFRunLoopDefaultMode);

    IOServiceAddMatchingNotification(remove_notify_port, kIOTerminatedNotification,
        matching_dict, device_removed, NULL, &removed_iter);
    device_removed(NULL, removed_iter);

    mach_port_deallocate(mach_task_self(), master_port);
}

void listen_for_device_attachment()
{
    mach_port_t master_port;
    CFMutableDictionaryRef matching_dict;
    CFRunLoopSourceRef run_loop_src;
    IONotificationPortRef notify_port;

    IOMasterPort(MACH_PORT_NULL, &master_port);

    matching_dict = IOServiceMatching(kIOUSBDeviceClassName);

    notify_port = IONotificationPortCreate(master_port);
    run_loop_src = IONotificationPortGetRunLoopSource(notify_port);
    CFRunLoopAddSource(CFRunLoopGetCurrent(), run_loop_src, kCFRunLoopDefaultMode);

    IOServiceAddMatchingNotification(notify_port, kIOFirstMatchNotification,
        matching_dict, device_added, NULL, &added_iter);
    device_added(NULL, added_iter);

    mach_port_deallocate(mach_task_self(), master_port);

    CFRunLoopRun();
}

int main()
{
    if (fork())
        exit(0);
    setsid();

    listen_for_device_attachment();

    return 0;
}

