#!/bin/sh
echo "Apple TV External Hard Disk Tool v1.20"
echo "Copyright (c) 2007 Patrick Walton <pcwalton@uchicago.edu> but freely"
echo "redistributable: see the COPYING file for details."
echo "Modifications and additions by Tom Anthony - AppleTVHacks.net."
echo "------------------------------------------------------------------------"
echo "Hi, I'm the install program for the Apple TV External Hard Disk tool."
echo "This tool allows you to seamlessly use an external USB drive as the "
echo "primary storage for your Apple TV. Please note that there is no warranty"
echo "for this tool."
echo ""
echo "Before proceeding, please ensure that you have all of the following:"
echo " - A connected Apple TV, modified at least with SSH capability. For"
echo "   information on how to modify your Apple TV to enable SSH, please see"
echo "   <http://www.AppleTVHacks.net/>."
echo " - An installed version of Mac OS X 10.4 Intel, or a full copy of the "
echo "   contents of the /System/Library/Extensions folder from one."
echo " - An original, unmodified copy of the 'mach_kernel.prelink' file from"
echo "   the Apple TV. I can try to retrieve it automatically from the Apple"
echo "   TV, but if you have modified the version on your Apple TV you'll need"
echo "   to tell me where I can find the original."
echo " - A copy of 'prelink_tool', which is located in the prelink_tool "
echo "   folder. The binary there is precompiled for Mac OS X Intel, but if"
echo "   you aren't running that you'll need to compile the included source"
echo "   yourself."
echo ""
read -e -p "Are you ready to install? [yes] " resp

if [[ $resp && $resp != "yes" ]]
then
    echo "Okay, not installing."
    exit
fi

echo "I need to know the IP address of your Apple TV. You can find this in the"
echo "Settings > Network menu on the Apple TV."
echo ""
read -e -p "What is the IP address of your Apple TV? " ip

echo "First thing I'll need to do is to get a copy of the kernel. If you have"
echo "a copy (called 'mach_kernel.prelink'), then I can use that, but it must"
echo "be unmodified (in particular, without Turbo's USB hack). If you don't"
echo "have it on your disk, then I can try to fetch it from the Apple TV, but"
echo "again it must be unmodified."
echo ""
read -e -p "Do you have a copy of the kernel on *this* computer? [no] " resp

if [[ $resp && $resp = "yes" ]]
then
    read -e -p "Where can I find it? [./mach_kernel.prelink] " compk
    if [[ !$compk ]]
    then
        compk=./mach_kernel.prelink
    fi
else
    echo "Okay, I'll get it from the Apple TV then. When prompted, enter "
    echo "'frontrow' (no quotes) as the password."
    scp -1 frontrow@$ip:/mach_kernel.prelink ./mach_kernel.prelink
    compk=./mach_kernel.prelink
fi

echo "Uncompressing the original '$compk'..."
./prelink_tool/prelink_tool -d $compk mach_kernel.ou.prelink
echo "Patching the original '$compk'..."
bspatch mach_kernel.ou.prelink mach_kernel.nu.prelink \
    kernel/kernel.uncompressed.patch || exit 1
echo "Compressing the new '$compk'..."
./prelink_tool/prelink_tool -e mach_kernel.nu.prelink mach_kernel.n.prelink

echo "I'm going to upload the patched kernel to the Apple TV now. When "
echo "prompted, which will likely happen several times, enter 'frontrow' (no "
echo "quotes) as the password."

ssh -1 frontrow@$ip "sudo mount -rw /dev/disk0s2 /; sudo mv /mach_kernel.prelink /mach_kernel.preusbhd.prelink" || exit 1
scp -1 ./mach_kernel.n.prelink \
    frontrow@$ip:/Users/frontrow/mach_kernel.n.prelink || exit 1
scp -1 ./mach_kernel.nu.prelink \
    frontrow@$ip:/Users/frontrow/mach_kernel.nu.prelink || exit 1
ssh -1 frontrow@$ip "sudo mv /Users/frontrow/mach_kernel.n.prelink /mach_kernel.prelink; sudo mv /Users/frontrow/mach_kernel.nu.prelink /mach" || exit 1

echo "Now I'll need to package up the necessary Mac OS X 10.4 kernel "
echo "extensions. If you're running Mac OS X 10.4 Intel, then they'll be"
echo "located in the system location, '/System/Library/Extensions', and you"
echo "can just press Return here. Otherwise, enter the location where they're"
echo "stored."
echo ""
read -e -p "Where are the kernel extensions? [/System/Library/Extensions] " \
    kextdir

if [[ !$kextdir ]]
then
    kextdir=/System/Library/Extensions
fi

thisdir=`pwd`
cd $kextdir
tar cf $thisdir/usbhd-kexts.tar IOStorageFamily.kext \
    IOSCSIArchitectureModelFamily.kext IOUSBMassStorageClass.kext || exit 1
cd $thisdir

echo "I need to send this package to your Apple TV now. When prompted, enter"
echo "'frontrow' for a password as before."

scp -1 usbhd-kexts.tar frontrow@$ip:/Users/frontrow/usbhd-kexts.tar
ssh -1 frontrow@$ip "sudo mv /Users/frontrow/usbhd-kexts.tar /System/Library/Extensions; sudo sh -c 'cd /System/Library/Extensions; tar xf usbhd-kexts.tar; chown -R root:wheel *'" || exit 1

echo "Now I need to install the 'atvrhdd' program, the 'atvrhdctrl' and "
echo "'backrow_patch' helper scripts onto your Apple TV. These enable your"
echo "Apple TV to automatically detect and use an external USB drive when "
echo "you insert one. When prompted, enter 'frontrow' as your password."
scp -1 /usr/bin/killall frontrow@$ip:/Users/frontrow/killall
scp -1 atvrhdctrl/atvrhdctrl.sh frontrow@$ip:/Users/frontrow/atvrhdctrl.sh
scp -1 backrow_patch/backrow_patch.pl frontrow@$ip:/Users/frontrow/backrow_patch.pl
scp -1 atvrhdd/atvrhdd frontrow@$ip:/Users/frontrow/atvrhdd

ssh -1 frontrow@$ip "sudo cp /Users/frontrow/killall /usr/bin/killall; sudo cp /Users/frontrow/atvrhdd /usr/bin/atvrhdd; sudo cp /Users/frontrow/atvrhdctrl.sh /usr/bin/atvrhdctrl.sh; sudo cp /Users/frontrow/backrow_patch.pl /usr/bin/backrow_patch.pl; sudo chmod a+x /usr/bin/killall /usr/bin/atvrhdd /usr/bin/atvrhdctrl.sh /usr/bin/backrow_patch.pl; sudo sh -c 'echo /usr/sbin/diskutil mountDisk disk1 >> /etc/rc.local'; sudo sh -c 'echo /usr/bin/atvrhdd >> /etc/rc.local'"

echo "Finally, I need to restart your Apple TV. When prompted, enter"
echo "'frontrow' as your password."
ssh -1 frontrow@$ip "sudo shutdown -r now"
echo "All done! Have fun."

